/***************************************************************************
                          graphics.c
 -  description: graphics items
                             -------------------
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Sam Hart
    email                : hart@geekcomix.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>

#include "globals.h"
#include "graphics.h"
#include "funcs.h"

Mix_Chunk      *sound[NUM_WAVES];
Mix_Music      *music;

/* Moved global so we can handle background
  scrolling outside of titlescreen() */
SDL_Surface    *background;
SDL_Surface    *bkg_src;

/*************************
	WaitFrame : Wait for
	the next frame.
**************************/
void WaitFrame(void)
{ //supposed to wait for next frame (not sure if it works, lifted from SDL demo aliens)
    static Uint32   next_tick = 0;
    Uint32          this_tick;

    /*
     * Wait for the next frame
     */
    this_tick = SDL_GetTicks();
    if (this_tick < next_tick) {
        SDL_Delay(next_tick - this_tick);
    }
    next_tick = this_tick + (1000 / FRAMES_PER_SEC);
}

int next_x(void)
{
	static double i = 0;

	i+=X_STEP;
	if(i > 200) i=0;
	return ((bkg_src->w / 2) + (bkg_src->w / 2) * sin(i));
}

int next_y(void)
{
	static double i = 0;

	i+=Y_STEP;
	if(i > 200) i=0;
	return ((bkg_src->h / 2) + (bkg_src->h / 2) * cos(i));
}

void UpdateTitleBackground(void)
{
	int x, y;
	int xx, yy;
	SDL_Rect src, dst;

    if(scroll != NO) {
        /* Upper-Left */
        x = next_x();
        y = next_y();

        src.x = x;
        src.y = y;
        src.w = bkg_src->w - src.x;
        src.h = bkg_src->h - src.y;

        dst.x = 0;
        dst.y = 0;
        dst.w = src.w;
        dst.h = src.h;
        SDL_BlitSurface(bkg_src, &src, background, &dst);

        if(x > 0) {
    		/* Upper right */
    		src.x = 0;
    		src.y = y;
    		src.w = x;
    		src.h = bkg_src->h - src.y;

    		dst.x = bkg_src->w - x;
    		dst.y = 0;
    		dst.w = src.w;
    		dst.h = src.h;
    		SDL_BlitSurface(bkg_src, &src, background, &dst);
    	}
    	if(y > 0) {
    		/* Lower left */
    		src.x = x;
    		src.y = 0;
    		src.w = bkg_src->w - x;
    		src.h = y;

      		dst.x = 0;
    		dst.y = bkg_src->h - y;
    		dst.h = src.h;
    		dst.w = src.w;
    		SDL_BlitSurface(bkg_src, &src, background, &dst);
    	}
    	if(x > 0 && y > 0) {
    		/* Lower right */
    		src.x = 0;
    		src.w = x;
    		src.y = 0;
    		src.h = y;

    		dst.x = bkg_src->w - x;
    		dst.y = bkg_src->h - y;
    		dst.w = src.w;
    		dst.h = src.h;
    		SDL_BlitSurface(bkg_src, &src, background, &dst);
    	}

        dst.w = background->w;
        dst.h = background->h;

    	for(xx = 0; xx < screen->w; xx+=background->w) {
    		for(yy = 0; yy < screen->h; yy+=background->h) {
    			dst.x = xx;
    			dst.y = yy;
    			SDL_BlitSurface(background, NULL, screen, &dst);
    		}
        }
    } else {
        dst.w = background->w;
        dst.h = background->h;

    	for(xx = 0; xx < screen->w; xx+=background->w) {
    		for(yy = 0; yy < screen->h; yy+=background->h) {
    			dst.x = xx;
    			dst.y = yy;
    			SDL_BlitSurface(bkg_src, NULL, screen, &dst);
    		}
        }
    }
}

/***************************
	LoadSound : Load a
	sound/music patch from a
	file.
****************************/
Mix_Chunk      *LoadSound(char *datafile, int verbose)
{ //load a sound from datafile
    Mix_Chunk      *temp;
    char            filename[FNLEN];
    int             i = 0;
    char            DATAPATH[FNLEN];
    int             local_use_theme = YES;

    strcpy(DATAPATH, PATH[0]);

    if (use_theme == YES) {
        sprintf(filename, "%s/sounds/%s", alt_theme_path, datafile);
    } else {
        sprintf(filename, "%s/sounds/%s", DATAPATH, datafile);
    }

    if (verbose == YES)
        printf( "LOADSOUND: Trying %s\n", filename);

    temp = Mix_LoadWAV(filename);

    while (temp == NULL) {
        strcpy(DATAPATH, PATH[i]);
        if (use_theme == YES && local_use_theme == YES) {
            sprintf(filename, "%s/%s/sounds/%s", DATAPATH, alt_theme_path,
                    datafile);
        } else {
            sprintf(filename, "%s/sounds/%s", DATAPATH, datafile);
        }

        if (verbose == YES)
            printf( "LOADSOUND: Trying %s\n", filename);

        temp = Mix_LoadWAV(filename);
        i++;

        if (i >= PATHNUM && sys_sound == YES) {
            if (local_use_theme == YES && use_theme == YES) {
                i = 0;
                local_use_theme = NO;
            } else {
                fprintf(stderr, "Couldn't load %s: %s\n", filename,
                        Mix_GetError());
                exit(2);
            }
        }
    }

    if (verbose == YES)
        printf( "LOADSOUND: Done\n");

    return (temp);
}

/************************
	LoadMusic : Load
	music from a datafile
*************************/
void LoadMusic(char *datafile, int verbose)
{ //Load the music from datafile
    char            filename[FNLEN];
    int             i = 0;
    char            DATAPATH[FNLEN];
    int             local_use_theme = YES;

    strcpy(DATAPATH, PATH[0]);

    if (use_theme == YES) {
        sprintf(filename, "%s/sounds/%s", alt_theme_path, datafile);
    } else {
        sprintf(filename, "%s/sounds/%s", DATAPATH, datafile);
    }

    if (verbose == YES)
        printf( "LOADMUSIC: Trying %s\n", filename);

    music = Mix_LoadMUS(filename);

    while (music == NULL) {
        strcpy(DATAPATH, PATH[i]);

        if (use_theme == YES && local_use_theme == YES) {
            sprintf(filename, "%s/%s/sounds/%s", DATAPATH, alt_theme_path,
                    datafile);
        } else {
            sprintf(filename, "%s/sounds/%s", DATAPATH, datafile);
        }

        if (verbose == YES)
            printf( "LOADMUSIC: Trying %s\n", filename);

        music = Mix_LoadMUS(filename);
        i++;

        if (i >= PATHNUM && sys_sound == YES) {
            if (local_use_theme == YES && use_theme == YES) {
                i = 0;
                local_use_theme = NO;
            } else {
                fprintf(stderr, "Couldn't load %s: %s\n", filename,
                        Mix_GetError());
                exit(2);
            }
        }
    }


    if (verbose == YES)
        printf( "LOADMUSIC: Done\n");
}

/*
 * Return the pixel value at (x, y)
 * NOTE: The surface must be locked before calling this!
 */
Uint32 getpixel(SDL_Surface *surface, int x, int y)
{
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to retrieve */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        return *p;

    case 2:
        return *(Uint16 *)p;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
            return p[0] << 16 | p[1] << 8 | p[2];
        else
            return p[0] | p[1] << 8 | p[2] << 16;

    case 4:
        return *(Uint32 *)p;

    default:
        return 0;       /* shouldn't happen, but avoids warnings */
    }
}


/***********************
	LoadImage : Load an
	image and set
	transparent if
	requested
************************/
SDL_Surface    *LoadImage(char *datafile, int transparent, int verbose)
{ //Load an image and set transparent (note, must use rgb = 255, 255, 0 as transparent color)
    SDL_Surface    *pic,
                   *pic2;
    char            filename[FNLEN];
    int             i = 0;
    char            DATAPATH[FNLEN];
    int             local_use_theme = YES;
    Uint32          key;

    /*verbose = YES; for forced debugging ;-)*/

    strcpy(DATAPATH, PATH[0]);

    if (use_theme == YES) {
        sprintf(filename, "%s/images/%s", alt_theme_path, datafile);
    } else {
        sprintf(filename, "%s/images/%s", DATAPATH, datafile);
    }

    if (verbose == YES)
        printf( "LOADIMAGE: Trying %s\n", filename);

    pic = IMG_Load(filename);

    if (verbose == YES)
        printf( "LOADIMAGE: testing for existance\n");

    while (pic == NULL) {
        strcpy(DATAPATH, PATH[i]);

        if (use_theme == YES && local_use_theme == YES) {
            sprintf(filename, "%s/%s/images/%s", DATAPATH, alt_theme_path,
                    datafile);
        } else {
            sprintf(filename, "%s/images/%s", DATAPATH, datafile);
        }

        if (verbose == YES)
            printf( "LOADIMAGE: Trying %s\n", filename);

        pic = IMG_Load(filename);
        i++;

        if (i >= PATHNUM) {
            if (local_use_theme == YES && use_theme == YES) {
                i = 0;
                local_use_theme = NO;
            } else {
                fprintf(stderr, "Couldn't load %s: %s\n", filename,
                        Mix_GetError());
                exit(2);
            }
        }
    }

    if (transparent) {
        /* Now we use the corner pixel for transparency */
        SDL_LockSurface(pic);
        key = getpixel(pic, 0, 0);
        SDL_UnlockSurface(pic);
        SDL_SetColorKey(pic, (SDL_SRCCOLORKEY | SDL_RLEACCEL), key);
    }
    pic2 = SDL_DisplayFormat(pic);
    SDL_FreeSurface(pic);

    if (verbose == YES)
        printf( "LOADIMAGE: Done\n");

    return (pic2);
}

/*********************
	StandbyScreen:
	Display the Standby
	screen.... duh
***********************/
void StandbyScreen()
{ //what to do when standing by ;)
    SDL_Surface    *standby;
    SDL_Rect        dest;
    char            filename[FNLEN];
    int             reset_theme = NO;

    strcpy(filename, "standby.png");

    reset_theme = use_theme;
    use_theme = NO; /* Sorry, I want the t4k logo not to be themed ;-) */
    standby = LoadImage(filename, 0, NO);
    use_theme = reset_theme;

    dest.x = ((screen->w) / 2) - (standby->w) / 2;
    dest.y = ((screen->h) / 2) - (standby->h) / 2;
    dest.w = standby->w;
    dest.h = standby->h;

    SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 0, 0, 0));
    SDL_BlitSurface(standby, NULL, screen, &dest);
    SDL_UpdateRect(screen, 0, 0, 0, 0);

    /*
     * Wait a while- give screen chance to come up (if fullscreen) 
     */
    SDL_Delay(WAIT_MS);

    SDL_FreeSurface(standby);

}

/*************************
	FreeGraphics: Free all
	the allocated graphics
**************************/
void FreeGraphics(int verbose)
{ // free allocd graphics
    if (verbose == YES) {
        printf( "FreeGraphics():\n");
        printf( "-Freeing screen\n");
    }
    SDL_FreeSurface(screen);
    if (verbose == YES)
        printf( "-Screen freed\n");

    if (verbose == YES) {
        printf( "FreeGraphics():END\n");
    }
}

/*************************
	TitleScreen: Display
	the title screen
**************************/
int TitleScreen(int verbose)
{ // display title screen, get input
    SDL_Surface    *temp,
                   *temp2;
    SDL_Surface    *title,
                   *cursor;
    SDL_Surface    *Tux[MAX_TUX_ANIM_FRAMES + 1];
    SDL_Surface    *menu[TITLE_MENU_ITEMS + 1][TITLE_MENU_DEPTH + 1];
    SDL_Rect        dest,
                    destd,
                    tempdest,
                    CursRect;
    SDL_Rect        Tuxdest,
                    Titledest;
    SDL_Rect        menudest[TITLE_MENU_ITEMS + 1];
    SDL_Rect        orig_menudest[TITLE_MENU_ITEMS + 1];
    char            filename[FNLEN];
    int             back_w,
                    back_h;
    int             i,
                    j,
                    tux_frame;
    int             x = 0;
    int             y = 0;
    int             dirx = 1;
    int             diry = 1;
    int             done = NO;
    int             menu_opt = MAIN;
    int             sub_menu = DONE;
    int             menu_depth = 1;
    int             last = 0;
    int             key_menu = 5;
    int             old_key_menu = 0;
    Uint8           state;
    Uint8          *keys;

    //setup menu items
    menu_item[1][1] = KCASCADE;
    menu_item[2][1] = WCASCADE;
    menu_item[3][1] = PRACTICE;
    menu_item[4][1] = DONE;

    menu_item[1][2] = LEVEL1;
    menu_item[2][2] = LEVEL2;
    menu_item[3][2] = LEVEL3;
    menu_item[4][2] = MAIN;

    menu_item[1][3] = PKCASCADE;
    menu_item[2][3] = PWCASCADE;
    menu_item[3][3] = PFREETYPE;
    menu_item[4][3] = MAIN;
    //done setup menu items (like to move in globals eventually)

    if (verbose == YES) {
        printf( "->TitleScreen():\n");
        printf( "->>Loading graphics\n");
    }
    //LOAD MAIN TITLE SCREEN IMAGES
    if (verbose == YES)
        printf( "->>checking for images\n");

    sprintf(filename, "title1.png");

    title = LoadImage(filename, 1, verbose);

    if (title == NULL) {
        fprintf(stderr, "!!Error Allocating Image Files!\n");
        exit(2);
    }

    if (verbose == YES) {
        printf( "->>Images found\n");
        printf( "->>Loading bkg image\n");
    }
    //strcpy(filename, "main_bkg.png");
    sprintf(filename, "main_bkg.png");
    bkg_src = LoadImage(filename, 0, verbose);
    if (bkg_src == NULL) {
        fprintf(stderr, "!!Error Allocating Image Files!\n");
        exit(2);
    }

    for (i = 1; i <= MAX_TUX_ANIM_FRAMES; i++) {
        //load tux animation
        if (verbose == YES)
            printf( "->>Loading tux anim frame %i\n", i);
        sprintf(filename, "tux%i.png", i);
        if (verbose == YES)
            printf( "->>Loading tux anim frame %s\n", filename);
        Tux[i] = LoadImage(filename, 1, verbose);
        if (Tux[i] == NULL) {
            fprintf(stderr, "!!Error Allocating Image Files!\n");
            exit(2);
        }
    }

    for (i = 1; i <= TITLE_MENU_ITEMS; i++)
        for (j = 1; j <= TITLE_MENU_DEPTH; j++) {
            //load menu items
            if (verbose == YES)
                printf( "->>Loading menu item %ix%i\n", j, i);
            sprintf(filename, "menu%i_%i.png", j, i);
            if (verbose == YES)
                printf( "->>Loading menu item %s\n", filename);
            menu[i][j] = LoadImage(filename, 1, verbose);
            if (menu[i][j] == NULL) {
                printf( "!!Error Allocating Image Files!\n");
                exit(2);
            }
        }

    if (verbose == YES)
        printf( "->>Loading cursor\n");

    if (hidden == YES)
        sprintf(filename, "cursorh.png");
    else
        sprintf(filename, "cursorn.png");

    cursor = LoadImage(filename, 1, verbose);
    if (cursor == NULL) {
        fprintf(stderr, "!!Error Loading Cursor file!\n");
        exit(2);
    }
    CursRect.w = cursor->w;
    CursRect.h = cursor->h;

    if (sys_sound == YES) {
        if (verbose == YES)
            printf( "->>Loading title music\n");

        if (hidden == YES)
            sprintf(filename, "funkbrk.it");
        else
            sprintf(filename, "funkbrk.it");

        LoadMusic(filename, verbose);
    }

    if (verbose == YES)
        printf( "->>Title images loaded, setting up background\n");

    //DONE LOADING TITLE IMAGES

    /*SETUP TITLE SCREEN BACKGROUND*/

    temp = SDL_CreateRGBSurface(SDL_HWSURFACE, bkg_src->w, bkg_src->h, 16,
		0, 0, 0, 0);
    background = SDL_DisplayFormat(temp);
    if(background == NULL) {
		fprintf(stderr, "CreateRGBSurface failed: %s\n", SDL_GetError());
		exit(1);
    }
    SDL_FreeSurface(temp);

    if (verbose == YES)
        printf(
                "->Background set up successfully, starting init anim\n");

    dest.x = 0;
    dest.y = 0;
    dest.w = screen->w;
    dest.h = screen->h;

    destd = dest;

    Tuxdest.x = 0;
    Tuxdest.y = screen->h - (Tux[1]->h / PRE_ANIM_FRAMES);
    Tuxdest.w = Tux[1]->w - (Tux[1]->w / PRE_ANIM_FRAMES);
    Tuxdest.h = Tux[1]->h / PRE_ANIM_FRAMES;



    Titledest.x = screen->w - ((TITLE_MULT * title->w) / PRE_ANIM_FRAMES);
    Titledest.y = (title->h) / 2;
    Titledest.w = (TITLE_MULT * title->w) / PRE_ANIM_FRAMES;
    Titledest.h = title->h;

    menudest[0].y = Titledest.y + (Titledest.h) / 2 + MENU_SEP;

    for (i = 1; i <= TITLE_MENU_ITEMS; i++) {
        menudest[i].x = ((screen->w) / 2) - (menu[i][1]->w) / 2;
        menudest[i].y = menudest[i - 1].y + MENU_SEP + menu[i][1]->h;
        menudest[i].w = menu[i][1]->w;
        menudest[i].h = menu[i][1]->h;
        orig_menudest[i] = menudest[i];
    }

    i = 0 - PRE_ANIM_FRAMES * PRE_FRAME_MULT;   // for first animation

    //for some reason, doesn't place mouse cursor where I want it
    //in full-screen.... but goofs SDL up enough to allow me to poll
    //the mouse's init position at 0,0.... which works fine to prevent

    //inital jerkiness of whatever menu item is at or near center of
    //screen!! Thus, fulfills its poip-pus! ;)
    SDL_WarpMouse(0, 0);

    //no cursor please
    SDL_ShowCursor(0);

    if (sys_sound == YES) {
        //Fade in music
        Mix_FadeInMusic(music, 1, FADE_IN_TITLE_MUSIC);
    }

    do {
        UpdateTitleBackground();
        i++;
        if (i == 0 && verbose == YES)
            printf( "->Begin Tux Anim Loop\n");

        if (sys_sound == YES) {
            //play the title music
            if (!Mix_PlayingMusic()) {
                Mix_PlayMusic(music, 1);
            }
        }
        //This PollEvent section was placed here largely to clean out
        //the event list between menu level changes (preventing unwanted double clicks)
        if (SDL_PollEvent(&event) > 0) {
            switch (event.type) {
            case SDL_KEYDOWN:
                {
                    if (event.key.keysym.sym == SDLK_ESCAPE) {
                        /*
                         * This quit event signals the closing of the window
                         */
                        if (verbose == YES)
                            printf(
                                    "->>Escape key rec'd from WM\n");
                        menu_opt = DONE;
                        done = YES;
                        i = FULL_CIRCLE + 1;
                    }
                    break;
                }
            case SDL_QUIT:
                {               /* This quit event signals the closing of the window */
                    if (verbose == YES)
                        printf( "->>Signal Quit rec'd from WM\n");
                    menu_opt = DONE;
                    done = YES;
                    i = FULL_CIRCLE + 1;
                    break;
                }
            case SDL_ACTIVEEVENT:
                {               /* For future usage */
                    if (verbose == YES)
                        printf( "->>App Focus has changed\n");
                }
            }
        }

        keys = SDL_GetKeyState(NULL);

        old_key_menu = key_menu;

        if (keys[SDLK_UP] == SDL_PRESSED) {
            if (last != 1) {
                key_menu--;
                last = 1;
            }
        } else if (keys[SDLK_DOWN] == SDL_PRESSED) {
            if (last != 2) {
                key_menu++;
                last = 2;
            }
        } else
            last = 0;

        if (key_menu < 1)
            key_menu = 4;
        if (key_menu > 4)
            key_menu = 1;

        if (old_key_menu != key_menu)
            SDL_WarpMouse(orig_menudest[key_menu].x +
                          (menudest[key_menu].w / 2),
                          orig_menudest[key_menu].y +
                          (2 * menudest[key_menu].h / 3));

        if (keys[SDLK_RETURN] == SDL_PRESSED) {
            while (keys[SDLK_RETURN] == SDL_PRESSED) {
                SDL_PollEvent(&event);
                keys = SDL_GetKeyState(NULL);
            }

            state = SDL_GetMouseState(&x, &y);
            if ((x >= orig_menudest[key_menu].x) &&
                (x <= (orig_menudest[key_menu].x + menudest[key_menu].w))
                && (y >= orig_menudest[key_menu].y)
                && (y <=
                    (orig_menudest[key_menu].y + menudest[key_menu].h))) {
                menu_opt = menu_item[key_menu][menu_depth];
                fprintf(stderr, "");    //WHAT THE ? This fixes segfault
                key_menu = 5;   // intential to cause update..
                old_key_menu = 0;
                i = FULL_CIRCLE + 1;
            }
        }


        if (i <= 0) {
            //do animations
            if (verbose == YES)
                printf( "->Doing beginning animations\n");

            tempdest.x = 0;
            tempdest.y = 0;
            tempdest.w = Tuxdest.w;
            tempdest.h = Tuxdest.h;
            SDL_BlitSurface(Tux[1], &tempdest, screen, &Tuxdest);
            tempdest.w = Titledest.w;
            tempdest.h = Titledest.h;
            SDL_BlitSurface(title, &tempdest, screen, &Titledest);

            Tuxdest.y -= Tux[1]->h / (PRE_ANIM_FRAMES * PRE_FRAME_MULT);
            Tuxdest.w += Tux[1]->w / (PRE_ANIM_FRAMES * PRE_FRAME_MULT);
            Tuxdest.h += Tux[1]->h / (PRE_ANIM_FRAMES * PRE_FRAME_MULT);

            if (Tuxdest.w >= Tux[1]->w || Tuxdest.h >= Tux[1]->h) {
                Tuxdest.w = Tux[1]->w;
                Tuxdest.h = Tux[1]->h;
            }

            Titledest.x -=
                (TITLE_MULT * title->w) / (PRE_ANIM_FRAMES *
                                           PRE_FRAME_MULT);
            Titledest.w +=
                (TITLE_MULT * title->w) / (PRE_ANIM_FRAMES *
                                           PRE_FRAME_MULT);
            if (Titledest.w >= title->w)
                Titledest.w = title->w;

        } else if (i <= FULL_CIRCLE) {
            if (i >= FULL_CIRCLE)
                i = 1;
            Tuxdest.y = screen->h - Tux[1]->h;
            done = NO;

            //ANIMATE TUX
            if (i >= TUX1 && i <= TUX2)
                tux_frame = 2;
            else if (i > TUX2 && i <= TUX3)
                tux_frame = 3;
            else if (i > TUX3 && i <= TUX4)
                tux_frame = 4;
            else if (i > TUX4 && i <= TUX5)
                tux_frame = 3;
            else if (i > TUX5 && i <= TUX6)
                tux_frame = 2;
            else
                tux_frame = 1;
            SDL_BlitSurface(Tux[tux_frame], NULL, screen, &Tuxdest);

            //place title
            SDL_BlitSurface(title, NULL, screen, &Titledest);

            state = SDL_GetMouseState(&x, &y);
            for (j = 1; j <= TITLE_MENU_ITEMS; j++) {
                //check to see if cursor inside for animation
                if ((x >= orig_menudest[j].x
                     && x <= (orig_menudest[j].x + menudest[j].w))
                    && (y >= orig_menudest[j].y
                        && y <= (orig_menudest[j].y + menudest[j].h))) {
                    key_menu = j;
                    menudest[j].x =
                        orig_menudest[j].x +
                        (MENU_GYRO_AMP * sin(i * MENU_GYRO_MULT));
                    menudest[j].y =
                        orig_menudest[j].y +
                        (MENU_GYRO_AMP * cos(i * MENU_GYRO_MULT));
                } else {
                    menudest[j].x = orig_menudest[j].x;
                    menudest[j].y = orig_menudest[j].y;
                }
                tempdest = menudest[j];
                tempdest.h = menu[j][menu_depth]->h;
                tempdest.w = menu[j][menu_depth]->w;
                SDL_BlitSurface(menu[j][menu_depth], NULL, screen,
                                &tempdest);
            }

            //check for mouse buttone depression
            if (state == SDL_BUTTON(1) || state == SDL_BUTTON(2)
                || state == SDL_BUTTON(3)) {    // ehh, we're easy... whatever mouse button you wanna through at me is fine
                for (j = 1; j <= TITLE_MENU_ITEMS; j++) {
                    if ((x >= orig_menudest[j].x
                         && x <= (orig_menudest[j].x + menudest[j].w))
                        && (y >= orig_menudest[j].y
                            && y <=
                            (orig_menudest[j].y + menudest[j].h))) {
                        menu_opt = menu_item[j][menu_depth];
                        i = FULL_CIRCLE + 1;
                        if (verbose == YES)
                            printf( "->>BUTTON CLICK\n");
                    }
                }
            }
        } else {
            if (verbose == YES)
                printf( "MENU SELECTED, PROCESSING\n");
            if (i >= FULL_CIRCLE + SLING_FRAMES) {
                i = 1;
                if (verbose == YES)
                    printf( "PROCESSING END!\n");
            }
            if (menu_opt == DONE) {
                done = YES;
            } else if (menu_opt == PRACTICE) {
                menu_depth = PRACTICE_SUBMENU;
                if (verbose == YES)
                    printf( "PRACTICE SUBMENU\n");
            } else if (menu_opt == KCASCADE) {
                menu_depth = KCASCADE_SUBMENU;
                sub_menu = KCASCADE;
                if (verbose == YES)
                    printf( "KCASCADE SUBMENU\n");
            } else if (menu_opt == WCASCADE) {
                menu_depth = WCASCADE_SUBMENU;
                sub_menu = WCASCADE;
                if (verbose == YES)
                    printf( "WCASCADE SUBMENU\n");
            } else if (menu_opt == MAIN) {
                menu_depth = ROOTMENU;
                if (verbose == YES)
                    printf( "ROOT MENU\n");
            } else if (menu_opt == LEVEL1) {
                done = YES;
                if (sub_menu == KCASCADE) {
                    menu_opt = KCASCADE1;
                } else {
                    menu_opt = WCASCADE1;
                }
            } else if (menu_opt == LEVEL2) {
                done = YES;
                if (sub_menu == KCASCADE) {
                    menu_opt = KCASCADE2;
                } else {
                    menu_opt = WCASCADE2;
                }
            } else if (menu_opt == LEVEL3) {
                done = YES;
                if (sub_menu == KCASCADE) {
                    menu_opt = KCASCADE3;
                } else {
                    menu_opt = WCASCADE3;
                }
            } else if (menu_opt == PKCASCADE) {
                done = YES;
            } else if (menu_opt == PFREETYPE) {
                done = YES;
            } else if (menu_opt == PWCASCADE) {
                done = YES;
            }

            menudest[0].y = Titledest.y + (Titledest.h) / 2 + MENU_SEP;
            for (j = 1; j <= TITLE_MENU_ITEMS; j++) {
                menudest[j].x =
                    ((screen->w) / 2) - (menu[j][menu_depth]->w) / 2;
                menudest[j].y =
                    menudest[j - 1].y + MENU_SEP + menu[j][menu_depth]->h;
                menudest[j].w = menu[j][menu_depth]->w;
                menudest[j].h = menu[j][menu_depth]->h;
                orig_menudest[j] = menudest[j];
            }

            //place tux
            SDL_BlitSurface(Tux[tux_frame], NULL, screen, &Tuxdest);

            //place title
            SDL_BlitSurface(title, NULL, screen, &Titledest);
        }

        //PLACE CURSOR
        if (x < 0)
            x = 0;
        if (x > (screen->w) - (cursor->w) - 1)
            x = (screen->w) - (cursor->w) - 1;

        if (y < 0)
            y = 0;
        if (y > (screen->h) - (cursor->h) - 1)
            y = (screen->h) - (cursor->h) - 1;

        CursRect.x = x;
        CursRect.y = y;
        SDL_BlitSurface(cursor, NULL, screen, &CursRect);
        //SDL_Flip(screen);
        if (speed_up == NO)
            WaitFrame();

        SDL_UpdateRect(screen, 0, 0, 0, 0);
    }
    while (done == NO);

    if (verbose == YES)
        printf( "->>Freeing title screen images\n");

    SDL_FreeSurface(background);
    SDL_FreeSurface(bkg_src);
    SDL_FreeSurface(title);

    for (i = 1; i <= MAX_TUX_ANIM_FRAMES; i++) {
        SDL_FreeSurface(Tux[i]);
    }

    for (i = 1; i <= TITLE_MENU_ITEMS; i++)
        for (j = 1; j <= TITLE_MENU_DEPTH; j++) {
            SDL_FreeSurface(menu[i][j]);
        }

    SDL_FreeSurface(cursor);

    if (sys_sound == YES) {
        //stop the music
        Mix_HaltMusic();
        Mix_FreeMusic(music);
    }

    if (verbose == YES)
        printf( "->TitleScreen():END\n");
    return menu_opt;
}                               //note to myself: font size in menu used was 38 pixels, GIMP logo "Comic Book", using "Blue_Green" gradient

/***************************
	GraphicsInit: Initializes
	the graphic system
****************************/
void GraphicsInit(Uint32 video_flags, int verbose)
{
    /*
     * Open the display device 
     */
    if (verbose == YES) {
        printf( "GraphicsInit():\n");
        printf( "-SDL Setting VidMode to %ix%ix%i\n", RES_X, RES_Y,
                BPP);
    }

    screen = SDL_SetVideoMode(RES_X, RES_Y, BPP, video_flags);
    if (screen == NULL) {
        fprintf(stderr, "Couldn't set %ix%i video mode: %s\n", RES_X,
                RES_Y, SDL_GetError());
        exit(2);
    }

    /*
     * Let's set the appropriate clip rect 
     */
    SDL_SetClipRect(screen, NULL);

    /*
     * no cursor please 
     */
    SDL_ShowCursor(0);

    StandbyScreen();

    /*
     * Set window manager stuff: 
     */
    SDL_WM_SetCaption("Tux Typing", PACKAGE);

    if (verbose == YES) {
        printf( "-SDL VidMode successfully set to %ix%ix%i\n",
                RES_X, RES_Y, BPP);
        printf( "GraphicsInit():END\n");
    }
}

/****************************
	LibInit : Init the SDL
	library
*****************************/
void LibInit(Uint32 lib_flags, int verbose)
{
    /*
     * Inizialize the SDL library 
     */
    if (verbose == YES) {
        printf( "LibInit():\n");
        printf( "-About to init SDL Library\n");
    }

    if (SDL_Init(lib_flags) < 0) {
        // First let's try without sound if sound is set
        if (sys_sound == YES) {
            lib_flags = (SDL_INIT_VIDEO);
            sys_sound = NO;
        }
        if (SDL_Init(lib_flags) < 0) {
            fprintf(stderr, "Couldn't initialize SDL: %s\n",
                    SDL_GetError());
            exit(2);
        }
    }

    if (verbose == YES)
        printf( "-SDL Library init'd successfully\n");

    /*
     * fire and forget... 
     */
    atexit(SDL_Quit);

    if (verbose == YES) {
        printf( "-SDL_Quit updated\n");
        printf( "LibInit():END\n");
    }
    SDL_EnableKeyRepeat(0, SDL_DEFAULT_REPEAT_INTERVAL);
}
